'**********************************************************************
'                            LED_MPLEX.BAS
'                         Flash of Inspiration I
'(c) W. Harth, DF2OAG
'(c) modified by Ing.Bro St. Lehmann, D-77756 Hausach, SL@iL-online.de
'**********************************************************************
rem      Demoprogramm fr LED-Anzeige im Multiplexbetrieb
rem             
rem      Einrichten der Platine
rem
rem      Tasten PortB -> offen
rem      LCD an PortB -> offen
rem      Beide LED-Anzeigen in die Fassung einsetzen
rem      Stift PortA,0 an Stift LD1
rem      Stift PortA,1 an Stift LD2
rem      Stift PortA,2 an Stift LS
rem
rem      Programm in PIC 16F628 laden und starten
rem
rem      Die LED-Anzeige beginnt bei 00 und zhlt im Sekundentakt
rem      bis 59 und beginnt wieder von vorn
rem      Jede Minute ertnt ein Signal
rem
rem      -------------------------------------------------------------
rem      demonstrates how to program a time multiplex led driver
rem
rem      setting and connections that must be done
rem
rem      switches "Tasten PortB" all off
rem      switsche "LCD an PortB" all off
rem      connect LD1 to PortA,0
rem      connect LD2 to PortA,1
rem      connect LS  to PortA,2
rem
rem      load compiled program into "Flash Of Inspiration I"
rem      
rem      The counter starts at 00 and counts up to 59 and turns back to 00
rem      each minute a beep is generated


         $NCALDEF
         
'Watchdog off, all comparators off, MCLR- and OSC2-Pin are used for io  
         define device 16f628
         config wdt_off
         config ircp_osc
         config cmcfg7
         config MCLR_INT
     
         xtal 4.00                     'xtal frequency
         
         tris ra,0                     'all pins are output now
         tris rb,0                     'all pins are output now
         output ra,0                   'all pins have low level
         output rb,0                   'all pins have low level

rem Variablen
'we use 9 variables stored on ram location 20h to 29h
'variable ZEIT is a word variable which needs two ram locations (22h low, 23h high) 
         define tmp1=$20     as byte
         define tmp2=$21     as byte
         define zeit=$22     as word       '16 Bit Variable
         define sieben=$24   as byte
         define wert=$25     as byte
         define wert1=$26    as byte
         define wert2=$27    as byte
         define sieben1=$28  as byte
         define sieben2=$29  as byte

'this are the entry of a cold start (power on or reset)
'we init the timer and some variables
         clock zeit                    'starts internal clock (with interrupt)
         let tmp1=0
         let tmp2=0
         let zeit=0
         let sieben1=%00111111         'pattern for character 0 on leds
         let sieben2=%00111111
         goto start                    'skip over all subroutines
       
'this subroutine generates the beep         
snd:
         if tmp2,0=1 then goto snd2
         sound ra,2,800,100
         set tmp2,0
snd2:
         return
         
'this subroutine converts the value in a 7 segment pattern
'umsetzen der Ziffern 0-9 in 7 Segment-code
'Eingabe: Ziffer in Variable            wert
'Ausgabe: Siebensegment in Variable     sieben
Ziffer:         
         if wert >9 then goto start            'Wert zu gro, starte neu
         on wert goto z0,z1,z2,z3,z4,z5,z6,z7,z8,z9        
z0:      ' Ziffer 0 und so weiter
         let sieben=%00111111
         goto ziffer_end
z1:
         let sieben=%00000110
         goto ziffer_end      
z2:
         let sieben=%01011011
         goto ziffer_end      
z3:
         let sieben=%01001111
         goto ziffer_end      
z4:
         let sieben=%01100110
         goto ziffer_end      
z5:
         let sieben=%01101101
         goto ziffer_end      
z6:
         let sieben=%01111100
         goto ziffer_end      
z7:
         let sieben=%00000111
         goto ziffer_end      
z8:
         let sieben=%01111111
         goto ziffer_end      
z9:
         let sieben=%01100111
         goto ziffer_end      
ziffer_end:
         return        
        
'here we turn on and off the 7 segmant display
mplex:
         set ra,0                      'left digit on
         output rb,sieben1             'pattern active
         wait 5                        'slow down to increase brightness
         res ra,0                      'left digit off
         set ra,1                      'right digit on
         output rb,sieben2             'pattern active
         wait 5                        'slow down to increase brightness
         res ra,1                      'right digit off 
         return
         
'this is the main loop
start:
         gosub mplex                   'call it often enough
         if zeit=60 then let zeit=0    'if we reached timers limit, set to 0
         if zeit=0 then gosub snd      'after 60 sec make a little bit noise
         if zeit=1 then res tmp2,0
         let wert=zeit                 'calculate 10th of seconds
         let wert1=wert/10             'it is a binary to decimal conversation
         gosub mplex                   'refresh dsplay so we get no flickering
         let tmp1=wert1*10
         let wert2=wert-tmp1
         let wert=wert1                'now we convert value to pattern
         gosub ziffer                  'for the first digit
         let sieben1=sieben            'save pattern result
         gosub mplex                   'refresh display
         let wert=wert2                'second digit
         gosub ziffer
         let sieben2=sieben            'save pattern result
         
         goto start                    'a never ending story
         

         