'*********************************************************************
'                            RTC_8583.BAS
'                         Flash of Inspiration I
'
'(c) W.Harth (DF2OAG)
'(c) modified by Ing.Bro St.Lehmann, D-77756 Hausach, SL@iL-online.de
'english commands by iL
'*********************************************************************
rem      Demoprogramm zum Ansprechen des Uhrenchip PCF 8583
rem
rem      Einrichten der Platine
rem      Tasten PortB -> S4 geschlossen
rem      LCD an PortB -> S3 glatt durch verbinden
rem      Stift PortA,1 an Stift SDA
rem      Stift PortA,2 an Stift SCL
rem
rem      Statusanzeige Jumper K36 alle entfernen
rem
rem      Programm in PIC 16F628 laden und starten
rem
rem      Zum Stellen der Uhrzeit, die Stern-Taste der Tastatur drcken.
rem      Sie knnen die Stunden und die Minuten stellen. Nach der Eingabe
rem      der Minuter-Einer, beginnt die Uhrzeit bei "00"-Sekunden zu laufen.
rem
rem      -------------------------------------------------------------------
rem      demo program shows how to program the rtc PCF8583 via I2C and some
rem      human interface
rem
rem      settings and connections
rem      switches "Tasten PortB" all on
rem      switches "LCD an PortB" all on
rem      connect SDA to PortA,1
rem      connect SCL to PortA,2
rem      all jumpers at K36 must be open
rem
rem      load compiled program in PIC 16F628
rem
rem      to set clock, press key "*"; enter hours and minutes
rem      after enter the last minute digit the clock starts with seconds 0



'Watchdog off, all comparators off, MCLR- and OSC2-Pin are used for io
              define device 16f628
              config wdt_off
              config irc_osc
              config cmcfg7
              config MCLR_int
     
              xtal 4.00 
              
              
rem init PIC
         tris ra,0           'all pins on Port A are outputs
         tris rb,0           'all pins on Port B are outputs

         output ra,0         'set all output levels to low
         output rb,0         'set all output levels to low
         
'tell the compiler about the presence of i2c bus, so it can initialize these 
'pins in the right manner          
         i2cinit ra,1,2                rem ra,1=SDA  ra,2=SCL         

         lcdinit rb,2,16,              'init. lcd with each 16 chars on 2 lines 
         
'conversation table for keypad. No keypressed returns "A", key 0 returns 11 etc.
'this table depends on how the keypad is connected to port 
         define keys rb,"A",11,7,4,1,0,8,5,2,12,9,6,3 
          
rem variablen und konstanten
'define variables and their ram locations
         define count  =$20 as word       
         define count1 =$20 as byte
         define count2 =$21 as byte
         define temp0  =$22 as byte
         define temp1  =$23 as byte
         define temp2  =$24 as byte
         define temp3  =$25 as byte
         define buffer =$26 as byte
         define temp4  =$27 as byte
         define temp5  =$28 as byte
         define b3     =$29 as byte
         define b4     =$30 as byte
         
         define  sek_einer =$40 as byte          rem 0. BCD-Byte
         define  sek_zehner=$41 as byte          rem 1. BCD-Byte
         define  min_einer =$42 as byte          rem 2. BCD-Byte
         define  min_zehner=$43 as byte          rem 3. BCD-Byte
         define  std_einer =$44 as byte          rem 4. BCD-Byte
         define  std_zehner=$45 as byte          rem 5. BCD-Byte
                      
         define  sek1=$47 as byte                rem Sekunde high-nibble=10er, low-nibble=1er--> zum Lesen
         define  min1=$48 as byte                rem Minuten high-nibble=10er, low-nibble=1er 
         define  std1=$49 as byte                rem Stunden high-nibble=10er, low-nibble=1er                   
         
rem init Peripherie
         clock count,500                         'start timer interrupts
         i2cwrite 160,0,%00000000                'config 8583 as real time clock
         lcdwrite 1,1,"PCF8583 Demo    "         'on power up or reset show this message
         lcdwrite 2,1,"                "

         wait 2000                               'display text for 2 seconds
         lcdwrite 1,1,"                "         'clear first line         
         goto start                              'skip all subroutines
         
'Uhrzeit fr die Ausgabe auf LCD vorbereiten
'converts bcd packed time format into ascii characters
anz1:
' seconds 1x and x1
         let temp0=sek1 and %00001111   'values from 0 to 9
         let temp1=sek1
         let sek_einer = temp0+48       'now it is an ascii character
         swap temp1
         let temp0=temp1 and %00000111  'values from 0 to 5
         let sek_zehner = temp0+48      'now it is an ascii character
'minutes 1x and x1        
         let temp0=min1 and %00001111   'values from 0 to 9
         let temp1=min1
         let min_einer = temp0+48       'now it is an ascii character
         swap temp1
         let temp0=temp1 and %00000111  'values from 0 to 5
         let min_zehner = temp0+48      'now it is an ascii character
'hours 1x and x1         
         let temp0=std1 and %00001111   'values from 0 to 9
         let temp1=std1
         let std_einer = temp0+48       'now it is an ascii character
         swap temp1
         let temp0=temp1 and %00000011  'values from 0 to 2
         let std_zehner = temp0+48      'now it is an ascii character
         return

'Ausgabe der Uhrzeit auf der LCD         
'writes the converted time string on the lcd
lcdschreiben:      
         lcdwrite 1,5,std_zehner,#,std_einer,#,":",min_zehner,#,min_einer,#,":",sek_zehner,#,sek_einer,#
         return

'Warte auf *-Taste zum Stellen der Uhr
'wait for '*' key to set new time
key:     
         inkey temp0
         if temp0 = 11 then goto stellen
         return 
         
'schreibt die rem hole Ziffer
taste:            
         lcdwrite 1,temp3,temp5,#
taste1:         
         wait 20
         if count1,0=1 then curson
         if count1,0=0 then cursoff  
         inkey temp0
         if temp0=65 then goto taste1
         cursoff
         return
         
'warte auf loslassen der Tastatur
'wait until all keys are released
taste2:  
         wait 20                       'debouncing
         inkey temp0
         if temp0=65 then return
         goto taste2
         
'Uhrzeit stellen                
'set new time
stellen: 
         lcdwrite 2,1,"               " 'clear second line
         i2cwrite 160,0,128             'we have to stop rtc 
         i2cwrite 160,2                 'set internal pointer to address 2
         i2cread  160,sek1,min1,std1    'read time
         
         let sek1=0                     'set seconds to 0
         gosub anz1                     'convert bcd packed time in ascii
         
         let temp3=4                    'location on lcd to edit
         let temp4=std_zehner           'hour in temp variable
         let temp5=32
       
         gosub lcdschreiben             'write time on lcd      
         
         gosub taste                    'get msb of hours from keypad
         if temp0 >=3 then goto stellen 'check valid range (0-2)
         swap temp0                     'swap nibbles
         let std1=std1 and %00001111    'set higher nibble to zero
         let std1=temp0 or std1         'replace higher nibble with new value
         gosub anz1                     'convert bcd to ascii
         gosub lcdschreiben             'display modified time
         gosub taste2                   'wait until all keys are released

stellen1:                               'now the same procedure for lsb of hours
         inc temp2
         inc temp3
         let temp5=std_zehner
stellen1a:
         gosub taste                    '1er Stunde holen
         if temp0 >=10 then goto stellen1a 
         let std1=std1 and %11110000
         let std1=std1 or temp0
         if std1>=36 then goto stellen  '36dez=24hex
         gosub anz1                     'daten fr LCD aufbereiten
         gosub lcdschreiben             'daten ausgebn
         gosub taste2                   'warte bis taste losgelassen wurde

stellen2:                               'now for X_ minutes
         inc temp3
         inc temp3
         let temp5=58                   '58dez = ":"
         
stellen2a:
         gosub taste                    '10er Minute holen
         if temp0 >=6 then goto stellen2a
         swap temp0
         let min1=min1 and %00001111
         let min1=temp0 or min1
               
         gosub anz1                     'daten fr LCD aufbereiten
         gosub lcdschreiben             'daten ausgebn
         gosub taste2                   'warte bis taste losgelassen wurde

stellen3:                               'now for _X minutes
         inc temp3
         let temp5=min_zehner
         
stellen3a:
         gosub taste                    '1er Minute holen
         if temp0 >=10 then goto stellen3a 
     
         let min1=min1 and %11110000
         let min1=min1 or temp0
     
         gosub anz1                     'daten fr LCD aufbereiten
         gosub lcdschreiben             'daten ausgebn
         gosub taste2                   'warte bis taste losgelassen wurde

'if the new time is entered, we must write it into the PCD8583
stellen_e:         
         i2cwrite 160,2,sek1,min1,std1
         i2cwrite 160,0,0              'clock starts again
         return
         
'main loop
start:
'Tastatur-Abfrage, warte auf *-Taste zum Stellen (Stern-taste hat den Wert 11)      
'key scan, waits until "*" (value 11) is pressed
         gosub key
'Sekunden, Minuten und Stunden aus PCF8583 auslesen      
'read seconds, minutes and hours from PCF8583
s1:
         i2cwrite 160,2                'set internal pointer to address 2
         i2cread  160,sek1,min1,std1   'read 3 ram locations (PCF8583 has autoincrement)
         gosub anz1                    'convert values for display
         gosub lcdschreiben            'write time on lcd     
         lcdwrite 2,1,"stellen mit *   "
         goto start                    'never ending story



       

        
